@@ -11,7 +11,12 @@ except ImportError:  # Airflow 2
 
 from airflow.models.base import ID_LEN as AIRFLOW_MAX_ID_LENGTH
 from airflow.models.dag import DAG
-from airflow.utils.task_group import TaskGroup
+
+try:
+    # Airflow 3.1 onwards
+    from airflow.sdk import TaskGroup
+except ImportError:
+    from airflow.utils.task_group import TaskGroup
 
 from cosmos import settings
 from cosmos.config import RenderConfig
@@ -19,6 +24,7 @@ from cosmos.constants import (
     DBT_SETUP_ASYNC_TASK_ID,
     DBT_TEARDOWN_ASYNC_TASK_ID,
     DEFAULT_DBT_RESOURCES,
+    PRODUCER_WATCHER_TASK_ID,
     SUPPORTED_BUILD_RESOURCES,
     TESTABLE_DBT_RESOURCES,
     DbtResourceType,
@@ -64,6 +70,24 @@ def calculate_operator_class(
     )
 
 
+def _is_source_used_by_filtered_nodes(source_node: DbtNode, filtered_nodes: dict[str, DbtNode]) -> bool:
+    """
+    Check if a source node is referenced by any of the filtered nodes.
+
+    :param source_node: The source node to check
+    :param filtered_nodes: Dictionary of filtered nodes
+    :returns: True if the source is used by any filtered node, False otherwise
+    """
+    source_id = source_node.unique_id
+
+    # Check if any filtered node depends on this source
+    for node in filtered_nodes.values():
+        if source_id in node.depends_on:
+            return True
+
+    return False
+
+
 def calculate_leaves(tasks_ids: list[str], nodes: dict[str, DbtNode]) -> list[str]:
     """
     Return a list of unique_ids for nodes that are not parents (don't have dependencies on other tasks).
@@ -152,9 +176,7 @@ def create_test_task_metadata(
     if test_indirect_selection != TestIndirectSelection.EAGER:
         task_args["indirect_selection"] = test_indirect_selection.value
     if node is not None:
-        if node.resource_type == DbtResourceType.MODEL:
-            task_args["models"] = node.resource_name
-        elif node.resource_type == DbtResourceType.SOURCE:
+        if node.resource_type == DbtResourceType.SOURCE:
             task_args["select"] = f"source:{node.resource_name}"
         elif is_detached_test(node):
             task_args["select"] = node.resource_name.split(".")[0]
@@ -163,7 +185,6 @@ def create_test_task_metadata(
 
         extra_context = {"dbt_node_config": node.context_dict}
         task_owner = node.owner
-
     elif render_config is not None:  # TestBehavior.AFTER_ALL
         task_args["select"] = render_config.select
         task_args["selector"] = render_config.selector
@@ -180,13 +201,22 @@ def create_test_task_metadata(
     if node:
         args_to_override = node.operator_kwargs_to_override
 
+    if (
+        execution_mode == ExecutionMode.WATCHER
+        and render_config is not None
+        and render_config.test_behavior == TestBehavior.AFTER_ALL
+    ):
+        operator_class = "cosmos.operators.local.DbtTestLocalOperator"
+    else:
+        operator_class = calculate_operator_class(
+            execution_mode=execution_mode,
+            dbt_class="DbtTest",
+        )
+
     return TaskMetadata(
         id=test_task_name,
         owner=task_owner,
-        operator_class=calculate_operator_class(
-            execution_mode=execution_mode,
-            dbt_class="DbtTest",
-        ),
+        operator_class=operator_class,
         arguments={**task_args, **args_to_override},
         extra_context=extra_context,
     )
@@ -196,7 +226,9 @@ def _get_task_id_and_args(
     node: DbtNode,
     args: dict[str, Any],
     use_task_group: bool,
+    execution_mode: ExecutionMode,
     normalize_task_id: Callable[..., Any] | None,
+    normalize_task_display_name: Callable[..., Any] | None,
     resource_suffix: str,
     include_resource_type: bool = False,
 ) -> tuple[str, dict[str, Any]]:
@@ -204,16 +236,26 @@ def _get_task_id_and_args(
     Generate task ID and update args with display name if needed.
     """
     args_update = args
-    task_display_name = f"{node.name}_{resource_suffix}"
+    task_name = f"{node.name}_{resource_suffix}"
+
     if include_resource_type:
-        task_display_name = f"{node.name}_{node.resource_type.value}_{resource_suffix}"
+        task_name = f"{node.name}_{node.resource_type.value}_{resource_suffix}"
+
     if use_task_group:
         task_id = resource_suffix
-    elif normalize_task_id:
+
+    elif normalize_task_id and not normalize_task_display_name:
         task_id = normalize_task_id(node)
-        args_update["task_display_name"] = task_display_name
+        args_update["task_display_name"] = task_name
+    elif not normalize_task_id and normalize_task_display_name:
+        task_id = task_name
+        args_update["task_display_name"] = normalize_task_display_name(node)
+    elif normalize_task_id and normalize_task_display_name:
+        task_id = normalize_task_id(node)
+        args_update["task_display_name"] = normalize_task_display_name(node)
     else:
-        task_id = task_display_name
+        task_id = task_name
+
     return task_id, args_update
 
 
@@ -249,12 +291,15 @@ def create_task_metadata(
     dbt_dag_task_group_identifier: str,
     use_task_group: bool = False,
     source_rendering_behavior: SourceRenderingBehavior = SourceRenderingBehavior.NONE,
+    source_pruning: bool = False,
     normalize_task_id: Callable[..., Any] | None = None,
+    normalize_task_display_name: Callable[..., Any] | None = None,
     test_behavior: TestBehavior = TestBehavior.AFTER_ALL,
     test_indirect_selection: TestIndirectSelection = TestIndirectSelection.EAGER,
     on_warning_callback: Callable[..., Any] | None = None,
     detached_from_parent: dict[str, DbtNode] | None = None,
     enable_owner_inheritance: bool | None = None,
+    filtered_nodes: dict[str, DbtNode] | None = None,
 ) -> TaskMetadata | None:
     """
     Create the metadata that will be used to instantiate the Airflow Task used to run the Dbt node.
@@ -273,7 +318,8 @@ def create_task_metadata(
     """
     dbt_resource_to_class = create_dbt_resource_to_class(test_behavior)
 
-    args = {**args, **{"models": node.resource_name}}
+    # Make a copy to avoid issues with mutable arguments
+    args = {**args}
 
     if DbtResourceType(node.resource_type) in DEFAULT_DBT_RESOURCES and node.resource_type in dbt_resource_to_class:
         extra_context: dict[str, Any] = {
@@ -281,18 +327,37 @@ def create_task_metadata(
             "dbt_dag_task_group_identifier": dbt_dag_task_group_identifier,
             "package_name": node.package_name,
         }
+        resource_suffix_map = {TestBehavior.BUILD: "build", DbtResourceType.MODEL: "run"}
+        resource_suffix = (
+            resource_suffix_map.get(test_behavior)
+            or resource_suffix_map.get(node.resource_type)
+            or node.resource_type.value
+        )
+        # Since Cosmos 1.11, it selects models using --select, instead of --models. The reason for this is that
+        # this flag was deprecated in dbt-core 1.10 (https://github.com/dbt-labs/dbt-core/issues/11561)
+        # and dbt fusion (2.0.0-beta26) does not support it.
+        # Users can still force Cosmos to use `--models` by setting the environment variable
+        # `AIRFLOW__COSMOS__PRE_DBT_FUSION=1`.
+        models_select_key = "models" if settings.pre_dbt_fusion else "select"
 
         if test_behavior == TestBehavior.BUILD and node.resource_type in SUPPORTED_BUILD_RESOURCES:
+            args[models_select_key] = f"{node.resource_name}"
             if test_indirect_selection != TestIndirectSelection.EAGER:
                 args["indirect_selection"] = test_indirect_selection.value
             args["on_warning_callback"] = on_warning_callback
             exclude_detached_tests_if_needed(node, args, detached_from_parent)
             task_id, args = _get_task_id_and_args(
-                node, args, use_task_group, normalize_task_id, "build", include_resource_type=True
+                node=node,
+                args=args,
+                use_task_group=use_task_group,
+                normalize_task_id=normalize_task_id,
+                normalize_task_display_name=normalize_task_display_name,
+                resource_suffix=resource_suffix,
+                include_resource_type=True,
+                execution_mode=execution_mode,
             )
-        elif node.resource_type == DbtResourceType.MODEL:
-            task_id, args = _get_task_id_and_args(node, args, use_task_group, normalize_task_id, "run")
         elif node.resource_type == DbtResourceType.SOURCE:
+            args["select"] = f"source:{node.resource_name}"
             args["on_warning_callback"] = on_warning_callback
 
             if (source_rendering_behavior == SourceRenderingBehavior.NONE) or (
@@ -301,9 +366,18 @@ def create_task_metadata(
                 and node.has_test is False
             ):
                 return None
-            args["select"] = f"source:{node.resource_name}"
-            args.pop("models")
-            task_id, args = _get_task_id_and_args(node, args, use_task_group, normalize_task_id, "source")
+
+            if source_pruning and filtered_nodes and not _is_source_used_by_filtered_nodes(node, filtered_nodes):
+                return None
+            task_id, args = _get_task_id_and_args(
+                node=node,
+                args=args,
+                use_task_group=use_task_group,
+                normalize_task_id=normalize_task_id,
+                normalize_task_display_name=normalize_task_display_name,
+                resource_suffix=r"source",
+                execution_mode=execution_mode,
+            )
             if node.has_freshness is False and source_rendering_behavior == SourceRenderingBehavior.ALL:
                 # render sources without freshness as empty operators
                 # empty operator does not accept custom parameters (e.g., profile_args). recreate the args.
@@ -312,9 +386,16 @@ def create_task_metadata(
                 else:
                     args = {}
                 return TaskMetadata(id=task_id, operator_class="airflow.operators.empty.EmptyOperator", arguments=args)
-        else:
+        else:  # DbtResourceType.MODEL, DbtResourceType.SEED and DbtResourceType.SNAPSHOT
+            args[models_select_key] = node.resource_name
             task_id, args = _get_task_id_and_args(
-                node, args, use_task_group, normalize_task_id, node.resource_type.value
+                node=node,
+                args=args,
+                use_task_group=use_task_group,
+                normalize_task_id=normalize_task_id,
+                normalize_task_display_name=normalize_task_display_name,
+                resource_suffix=resource_suffix,
+                execution_mode=execution_mode,
             )
 
         _override_profile_if_needed(args, node.profile_config_to_override)
@@ -363,10 +444,13 @@ def generate_task_or_group(
     test_behavior: TestBehavior,
     source_rendering_behavior: SourceRenderingBehavior,
     test_indirect_selection: TestIndirectSelection,
-    on_warning_callback: Callable[..., Any] | None,
+    source_pruning: bool = False,
+    on_warning_callback: Callable[..., Any] | None = None,
     normalize_task_id: Callable[..., Any] | None = None,
+    normalize_task_display_name: Callable[..., Any] | None = None,
     detached_from_parent: dict[str, DbtNode] | None = None,
     enable_owner_inheritance: bool | None = None,
+    filtered_nodes: dict[str, DbtNode] | None = None,
     **kwargs: Any,
 ) -> BaseOperator | TaskGroup | None:
     task_or_group: BaseOperator | TaskGroup | None = None
@@ -385,12 +469,15 @@ def generate_task_or_group(
         dbt_dag_task_group_identifier=_get_dbt_dag_task_group_identifier(dag, task_group),
         use_task_group=use_task_group,
         source_rendering_behavior=source_rendering_behavior,
+        source_pruning=source_pruning,
         normalize_task_id=normalize_task_id,
+        normalize_task_display_name=normalize_task_display_name,
         test_behavior=test_behavior,
         test_indirect_selection=test_indirect_selection,
         on_warning_callback=on_warning_callback,
         detached_from_parent=detached_from_parent,
         enable_owner_inheritance=enable_owner_inheritance,
+        filtered_nodes=filtered_nodes,
     )
 
     # In most cases, we'll  map one DBT node to one Airflow task
@@ -468,6 +555,71 @@ def _add_dbt_setup_async_task(
     tasks_map[DBT_SETUP_ASYNC_TASK_ID] = setup_airflow_task
 
 
+def _add_producer_watcher_and_dependencies(
+    dag: DAG,
+    task_args: dict[str, Any],
+    tasks_map: dict[str, Any],
+    task_group: TaskGroup | None,
+    render_config: RenderConfig | None = None,
+    nodes: dict[str, DbtNode] | None = None,
+) -> str:
+    producer_task_args = task_args.copy()
+
+    if render_config is not None:
+        producer_task_args["select"] = render_config.select
+        producer_task_args["selector"] = render_config.selector
+        producer_task_args["exclude"] = render_config.exclude
+
+        if render_config.test_behavior in [TestBehavior.NONE, TestBehavior.AFTER_ALL]:
+            producer_task_args["exclude"] = producer_task_args["exclude"] + [
+                "resource_type:test",
+                "resource_type:unit_test",
+            ]
+
+    # First, we create the producer task
+    producer_task_metadata = TaskMetadata(
+        id=PRODUCER_WATCHER_TASK_ID,
+        operator_class="cosmos.operators.watcher.DbtProducerWatcherOperator",
+        arguments=producer_task_args,
+    )
+    producer_airflow_task = create_airflow_task(producer_task_metadata, dag, task_group=task_group)
+
+    # Second, we need to set the producer task ID in all consumer tasks (and their children tasks)
+    for node_id, task_or_taskgroup in tasks_map.items():
+
+        node_tasks = (
+            list(task_or_taskgroup.children.values())
+            if isinstance(task_or_taskgroup, TaskGroup)
+            else [task_or_taskgroup]
+        )
+        for task in node_tasks:
+            task.producer_task_id = producer_airflow_task.task_id  # type: ignore[attr-defined]
+
+        # Third, we want to make the producer task to be the parent of the root dbt nodes, without blocking them from sensing XCom
+        # We only managed to do this in the case of DbtDag.
+        # The way it is implemented is by setting the trigger_rule to "always" for the consumer tasks, and by having the producer task with a high priority_weight.
+        if "DbtDag" in dag.__class__.__name__:
+
+            # Is this dbt node a root of the (subset of the) dbt project?
+            # Note: this may happen in one scenarios:
+            # - the dbt node not having any `depends_on` within the user-selected `nodes`
+            if nodes and node_id in nodes and not set(nodes[node_id].depends_on).intersection(nodes):
+                producer_airflow_task >> task_or_taskgroup
+                if isinstance(task_or_taskgroup, TaskGroup):
+                    taskgroup = task_or_taskgroup
+                    always_run_tasks = [
+                        task for task in node_tasks if not set(task.upstream_task_ids).intersection(taskgroup.children)
+                    ]
+                else:
+                    always_run_tasks = [task_or_taskgroup]
+
+                for task in always_run_tasks:
+                    task.trigger_rule = task_args.get("trigger_rule", "always")  # type: ignore[attr-defined]
+
+    tasks_map[PRODUCER_WATCHER_TASK_ID] = producer_airflow_task
+    return producer_airflow_task.task_id
+
+
 def should_create_detached_nodes(render_config: RenderConfig) -> bool:
     """
     Decide if we should calculate / insert detached nodes into the graph.
@@ -555,7 +707,7 @@ def _add_teardown_task(
     tasks_map[DBT_TEARDOWN_ASYNC_TASK_ID] = teardown_airflow_task
 
 
-def build_airflow_graph(
+def build_airflow_graph(  # noqa: C901 TODO: https://github.com/astronomer/astronomer-cosmos/issues/1943
     nodes: dict[str, DbtNode],
     dag: DAG,  # Airflow-specific - parent DAG where to associate tasks and (optional) task groups
     execution_mode: ExecutionMode,  # Cosmos-specific - decide what which class to use
@@ -594,7 +746,9 @@ def build_airflow_graph(
     node_converters = render_config.node_converters or {}
     test_behavior = render_config.test_behavior
     source_rendering_behavior = render_config.source_rendering_behavior
+    source_pruning = render_config.source_pruning
     normalize_task_id = render_config.normalize_task_id
+    normalize_task_display_name = render_config.normalize_task_display_name
     enable_owner_inheritance = render_config.enable_owner_inheritance
     tasks_map: dict[str, Union[TaskGroup, BaseOperator]] = {}
     task_or_group: TaskGroup | BaseOperator
@@ -605,6 +759,10 @@ def build_airflow_graph(
     detached_from_parent: dict[str, list[DbtNode]] = defaultdict(list)
     identify_detached_nodes(nodes, render_config, detached_nodes, detached_from_parent)
 
+    virtualenv_dir = None
+    if execution_mode == ExecutionMode.AIRFLOW_ASYNC:
+        virtualenv_dir = task_args.pop("virtualenv_dir", None)
+
     for node_id, node in nodes.items():
         conversion_function = node_converters.get(node.resource_type, generate_task_or_group)
         if conversion_function != generate_task_or_group:
@@ -621,12 +779,15 @@ def build_airflow_graph(
             task_args=task_args,
             test_behavior=test_behavior,
             source_rendering_behavior=source_rendering_behavior,
+            source_pruning=source_pruning,
             test_indirect_selection=test_indirect_selection,
             on_warning_callback=on_warning_callback,
             normalize_task_id=normalize_task_id,
+            normalize_task_display_name=normalize_task_display_name,
             node=node,
             detached_from_parent=detached_from_parent,
             enable_owner_inheritance=enable_owner_inheritance,
+            filtered_nodes=nodes,
         )
         if task_or_group is not None:
             logger.debug(f"Conversion of <{node.unique_id}> was successful!")
@@ -666,11 +827,22 @@ def build_airflow_graph(
             tasks_map[node_id] = test_task
 
     create_airflow_task_dependencies(nodes, tasks_map)
+
+    if execution_mode == ExecutionMode.WATCHER:
+        _add_producer_watcher_and_dependencies(
+            dag=dag,
+            task_args=task_args,
+            tasks_map=tasks_map,
+            task_group=task_group,
+            render_config=render_config,
+            nodes=nodes,
+        )
+
     if settings.enable_setup_async_task:
         _add_dbt_setup_async_task(
             dag,
             execution_mode,
-            task_args,
+            {**task_args, "virtualenv_dir": virtualenv_dir},
             tasks_map,
             task_group,
             render_config=render_config,
